#ifndef MECHBCLC_H
#define MECHBCLC_H

#include "iotools.h"
#include "alias.h"

class inicd;
class loadcase;
class dloadcase;
class gfunct;

/**
  Class mechbclc:
   
  It is one of the 5 most important classes of the program.
  (probdesc, mechtop, mechmat, mechbclc, mechcrsec)
   
  Class mechbclc contains data about boundary conditions, load cases.
   
  Created by JK, TKo
*/
class mechbclc
{
 public:
  mechbclc ();
  ~mechbclc ();
  void read (XFILE *in);
  void print (FILE *out);
  void read_eigenstrains (XFILE *in);
  void print_eigenstrains (FILE *out);
  void eigstrain_computation (double time);
  long readinic (XFILE *in);
  long printinic (FILE *out);
  void inicipval(void);
  void alloc_sumcomp ();
  void comp_sum (double *rhs);
  void give_comp_sum (double *sum);
  void comp_sum_react();
  void comp_sum_pdreact();

  /// number of load cases
  long nlc;
  /// number of initcond
  long nico;
  
  ///  number of components of array sumcomp
  long ncsum;
  ///  array containing sums of components of load %vector in particular directions 
  double *sumcomp;
  ///  array containing sums of components of reactions at supports in particular directions 
  double *reactsumcomp;
  ///  array containing sums of components of reactions at prescribed displacements in particular directions 
  double *pd_reactsumcomp;
  
  /// array of static load cases
  loadcase  *lc;
  /// array of time dependent load cases
  dloadcase *dlc;
  /// array of initial conditions
  inicd     *ico;
  
  ///  the number of general functions describing the eigenstrains
  long ngfes;
  ///  array of general functions describing the eigenstrains
  gfunct *eigstrfun;
  ///  stress-strain state
  strastrestate strastre;

    
};

#endif
