#include "shellq.h"
#include "global.h"
#include "globmat.h"
#include "genfile.h"
#include "node.h"
#include "element.h"
#include "plelemrotlq.h"
#include "q4plate.h"
#include "intpoints.h"
#include <math.h>

shellq::shellq (void)
{
  long i,j;
  
  if (Perlq==NULL)   Perlq = new planeelemrotlq;
  if (Q4pl==NULL)    Q4pl = new q4plate;
  
  //  number of nodes on element
  nne=4;
  //  number of DOFs on element
  ndofe=24;
  //  number of strain/stress components
  tncomp=6;
  //  number of functions approximated
  napfun=6;
  //  number of edges on element
  ned=4;
  //  number of nodes on one edge
  nned=2;
  //  strain/stress state
  ssst=shell;
  
  nb = Perlq->nb + Q4pl	->nb;

  ncomps = new long [Perlq->nb];
  ncomps[0]=2;
  ncomps[1]=1;
 
  ncompd = new long [Q4pl->nb];
  ncompd[0]=3;

  nip = new long* [nb];
  intordsm = new long* [nb];
  for (i=0;i<nb;i++){
    nip[i]=new long [nb];
    intordsm[i] = new long [nb];
  }
  tncomps=Perlq->tncomp;
  tncompd=Q4pl->tncomp;
  
  long a00,a01,a10,a11,b00,b01,b10,b11;

  a00=Perlq->nip[0][0];  a01=Perlq->nip[0][1];
  a10=Perlq->nip[1][0];  a11=Perlq->nip[1][1];
  b00=Q4pl->nip[0][0];  b01=Q4pl->nip[0][1];
  b10=Q4pl->nip[1][0];  b11=Q4pl->nip[1][1];

  nip[0][0]=a00;  nip[0][1]=a01;  nip[0][2]=0;   nip[0][3]=0;    
  nip[1][0]=a10;  nip[1][1]=a11;  nip[1][2]=0;   nip[1][3]=0;
  nip[2][0]=0;    nip[2][1]=0;    nip[2][2]=b00; nip[2][3]=b01;
  nip[3][0]=0;    nip[3][1]=0;    nip[3][2]=b10; nip[3][3]=b11;
  
  a00=Perlq->intordsm[0][0];  a01=Perlq->intordsm[0][1];
  a10=Perlq->intordsm[1][0];  a11=Perlq->intordsm[1][1];
  b00=Q4pl->intordsm[0][0];  b01=Q4pl->intordsm[0][1];
  b10=Q4pl->intordsm[1][0];  b11=Q4pl->intordsm[1][1];
  
  intordsm[0][0]=a00;  intordsm[0][1]=a01; intordsm[0][2]=0;   intordsm[0][3]=0;
  intordsm[1][0]=a10;  intordsm[1][1]=a11; intordsm[1][2]=0;   intordsm[1][3]=0;
  intordsm[2][0]=0;    intordsm[2][1]=0;   intordsm[2][2]=b00; intordsm[2][3]=b01;
  intordsm[3][0]=0;    intordsm[3][1]=0;   intordsm[3][2]=b10; intordsm[3][3]=b11;

  dofe = new long [2];
  dofe[0]=Perlq->ndofe;
  dofe[1]=Q4pl->ndofe;

  tnip=0;
  for (i=0;i<nb;i++){
    for (j=0;j<nb;j++){
      tnip+=nip[i][j];
    }
  }
}

shellq::~shellq (void)
{
  long i;
  
  for (i=0;i<nb;i++){
    delete [] nip[i];
    delete [] intordsm[i];
  }
  delete [] intordsm;
  delete [] nip;
  delete [] ncomps;
  delete [] ncompd;
  delete [] dofe; 
}



/**
   function assembles transformation matrix between coordinate system
   defined on element and global coordinate system
   transformation deals with coordinates
   
   transformation matrix from local to global systems x,y,z
   In vectors   s  are global coordinates of Nodes 1 , 2 , 3
   Vector lokal s. s.( edge 1-2 = x' )  in glob. s. s.
   
   v(g)  = tran * v(l)
               t                t
   v(l)  = tran  * v(g)  =  v(g)  * tran
   
                               t
   sm(g) = tran  * sm(l) * tran
               t
   sm(l) = tran  * sm(g) * tran
   
*/
void shellq::tran_mat(vector &x, vector &y, matrix &tran, vector &gx, vector &gy, vector &gz)
{
  long i,i1;
  double dl;
  matrix a(3,3);
 
  for (i=0; i<3; i++) {
    i1=i+1; if(i1>2)i1=i1-3;
//    a[i][0]=s2[i]-s1[i];
    a[0][i]=gx[i1]-gx[i];
//    a[i][1]=s3[i]-s2[i];
    a[1][i]=gy[i1]-gy[i];
//    a[i][2]=s1[i]-s3[i];
    a[2][i]=gz[i1]-gz[i];
  }
  
  dl=sqrt(a[0][0]*a[0][0]+a[1][0]*a[1][0]+a[2][0]*a[2][0]);

  tran[0][0]=a[0][0]/dl;
  tran[1][0]=a[1][0]/dl;
  tran[2][0]=a[2][0]/dl;
  
  tran[0][2]=a[1][0]*a[2][1]-a[2][0]*a[1][1];
  tran[1][2]=a[2][0]*a[0][1]-a[0][0]*a[2][1];
  tran[2][2]=a[0][0]*a[1][1]-a[1][0]*a[0][1];

  dl=sqrt(tran[0][2]*tran[0][2]+tran[1][2]*tran[1][2]+tran[2][2]*tran[2][2]);

  tran[0][2]=tran[0][2]/dl;
  tran[1][2]=tran[1][2]/dl;
  tran[2][2]=tran[2][2]/dl;
  
  tran[0][1]=tran[1][2]*tran[2][0]-tran[2][2]*tran[1][0];
  tran[1][1]=tran[2][2]*tran[0][0]-tran[0][2]*tran[2][0];
  tran[2][1]=tran[0][2]*tran[1][0]-tran[1][2]*tran[0][0];

  dl=sqrt(tran[0][1]*tran[0][1]+tran[1][1]*tran[1][1]+tran[2][1]*tran[2][1]);

  tran[0][1]=tran[0][1]/dl;
  tran[1][1]=tran[1][1]/dl;
  tran[2][1]=tran[2][1]/dl;
/*  
// local coordinate x=s12
    sl1[0]=0.0;    sl1[1]=0.0;    sl1[2]=0.0;
    sl2[0]=sqrt(a[0][0]*a[0][0]+a[1][0]*a[1][0]+a[2][0]*a[2][0]);
    sl2[1]=0.0;    sl2[2]=0.0;
    sl3[0]=(s3(0)-s1(0))*tran(0,0)+(s3(1)-s1(1))*tran(1,0)+(s3(2)-s1(2))*tran(2,0);
    sl3[1]=(s3(0)-s1(0))*tran(0,1)+(s3(1)-s1(1))*tran(1,1)+(s3(2)-s1(2))*tran(2,1);
    sl3[2]=0.0;
*/
// local coordinate x=s12
    x[0]=0.0;
    y[0]=0.0;
    x[1]=sqrt(a[0][0]*a[0][0]+a[1][0]*a[1][0]+a[2][0]*a[2][0]);
    y[1]=0.0;
    x[2]=(gx(2)-gx(0))*tran(0,0)+(gy(2)-gy(0))*tran(1,0)+(gz(2)-gz(0))*tran(2,0);
    y[2]=(gx(2)-gx(0))*tran(0,1)+(gy(2)-gy(0))*tran(1,1)+(gz(2)-gz(0))*tran(2,1);
    x[3]=(gx(3)-gx(0))*tran(0,0)+(gy(3)-gy(0))*tran(1,0)+(gz(3)-gz(0))*tran(2,0);
    y[3]=(gx(3)-gx(0))*tran(0,1)+(gy(3)-gy(0))*tran(1,1)+(gz(3)-gz(0))*tran(2,1);
}



/**
   function assembles transformation matrix between global coordinate system
   and local nodal coordinate system
   transformation deals with nodal forces

   transformation matrix x_g = T x_l

   9.5.2002
*/
void shellq::transf_matrix (ivector &nodes,matrix &tmat)
{
  long i,i6,n,m;

  fillm (0.0,tmat);

  n=nodes.n;
  m=tmat.m;
  for (i=0;i<m;i++){
    tmat[i][i]=1.0;
  }
  
  for (i=0;i<n;i++){
    if (Mt->nodes[nodes[i]].transf>0){
	  i6=i*6;
      tmat[i6][i6]   = Mt->nodes[nodes[i]].e1[0];  tmat[i6][i6+1]   = Mt->nodes[nodes[i]].e2[0]; tmat[i6][i6+2]     = Mt->nodes[nodes[i]].e3[0];
      tmat[i6+1][i6] = Mt->nodes[nodes[i]].e1[1];  tmat[i6+1][i6+1] = Mt->nodes[nodes[i]].e2[1]; tmat[i6+1][i6+2]   = Mt->nodes[nodes[i]].e3[1];
      tmat[i6+2][i6] = Mt->nodes[nodes[i]].e1[2];  tmat[i6+2][i6+1] = Mt->nodes[nodes[i]].e2[2]; tmat[i6+2][i6+2]   = Mt->nodes[nodes[i]].e3[2];
	  i6=i*6+3;
      tmat[i6][i6]   = Mt->nodes[nodes[i]].e1[0];  tmat[i6][i6+1]   = Mt->nodes[nodes[i]].e2[0]; tmat[i6][i6+2]     = Mt->nodes[nodes[i]].e3[0];
      tmat[i6+1][i6] = Mt->nodes[nodes[i]].e1[1];  tmat[i6+1][i6+1] = Mt->nodes[nodes[i]].e2[1]; tmat[i6+1][i6+2]   = Mt->nodes[nodes[i]].e3[1];
      tmat[i6+2][i6] = Mt->nodes[nodes[i]].e1[2];  tmat[i6+2][i6+1] = Mt->nodes[nodes[i]].e2[2]; tmat[i6+2][i6+2]   = Mt->nodes[nodes[i]].e3[2];
    }
  }
}


/**
   function assembles stiffness matrix of q shell element
   
   @param eid - element id
   @param sm - stiffness matrix
   
   4.5.2002
*/
void shellq::res_stiffness_matrix (long eid,matrix &sm)
{
  long *rcn,*ccn,transf;
  vector gx(nne),gy(nne),gz(nne),x(nne),y(nne);
  ivector nodes(nne);
  matrix lsm,tran(3,3);
  
  Mt->give_elemnodes (eid,nodes);
  Mt->give_node_coord3d (gx,gy,gz,eid);
  //  funkce transformujici x,y,z do roviny xy 
  tran_mat(x,y, tran, gx,  gy,  gz);
  
  fillm (0.0,sm);
  
  rcn = new long [dofe[0]];
  ccn = new long [dofe[0]];
  reallocm (dofe[0],dofe[0],lsm);
  
  //  contribution from q plane element with rotational degrees of freedom
  Perlq->stiffness_matrix (eid,0,0,lsm,x,y);
  /*
  for (int i=0;i<dofe[0];i++){
	  lsm[i][i]=1.0;
  }
  */

  rcn[0]=1;  rcn[1]=2;  rcn[2]=6;  rcn[3]=7;  rcn[4]=8;  rcn[5]=12;  rcn[6]=13;  rcn[7]=14;  rcn[8]=18;  rcn[9]=19;  rcn[10]=20;  rcn[11]=24;
  ccn[0]=1;  ccn[1]=2;  ccn[2]=6;  ccn[3]=7;  ccn[4]=8;  ccn[5]=12;  ccn[6]=13;  ccn[7]=14;  ccn[8]=18;  ccn[9]=19;  ccn[10]=20;  ccn[11]=24;
  
  mat_localize (sm,lsm,rcn,ccn);

  delete [] ccn;  delete [] rcn;
  
  rcn = new long [dofe[1]];
  ccn = new long [dofe[1]];
  reallocm (dofe[1],dofe[1],lsm);
  
//  contribution from q plate element
  Q4pl->stiffness_matrix (eid,2,2,lsm,x,y);
//  for (int i=0;i<dofe[1];i++){
//    fprintf (Out,"\n %10.7e %10.7e %10.7e %10.7e %10.7e %10.7e",lsm[i][0],lsm[i][1],lsm[i][2],lsm[i][3],lsm[i][4],lsm[i][5]);
//    fprintf (Out,"\n %10.7e %10.7e %10.7e %10.7e %10.7e %10.7e",lsm[i][6],lsm[i][7],lsm[i][8],lsm[i][9],lsm[i][10],lsm[i][11]);
//  }
  
  rcn[0]=3;  rcn[1]=4;  rcn[2]=5;  rcn[3]=9;  rcn[4]=10;  rcn[5]=11;  rcn[6]=15;  rcn[7]=16;  rcn[8]=17;  rcn[9]=21;  rcn[10]=22;  rcn[11]=23;
  ccn[0]=3;  ccn[1]=4;  ccn[2]=5;  ccn[3]=9;  ccn[4]=10;  ccn[5]=11;  ccn[6]=15;  ccn[7]=16;  ccn[8]=17;  ccn[9]=21;  ccn[10]=22;  ccn[11]=23;

  mat_localize (sm,lsm,rcn,ccn);
  //  transformation of stiffness matrix to GSS
  lgmatrixtransfblock(sm, tran);
  //  transformation of stiffness matrix to nodesystem
  transf = Mt->locsystems (nodes);
  if (transf>0){
    matrix tmat (ndofe,ndofe);
    transf_matrix (nodes,tmat);
    glmatrixtransf (sm,tmat);
  }
  delete [] ccn;  delete [] rcn;
  
}

  
void shellq::res_ip_strains (long lcid,long eid)
{
  long *rcn,i;
  vector aux;
  vector gx(nne),gy(nne),gz(nne),x(nne),y(nne),r(ndofe),rs(dofe[0]),rd(dofe[1]);
  ivector nodes(nne);
  matrix lsm,tmat,tran(3,3);
  
  Mt->give_node_coord3d (gx,gy,gz,eid);
  Mt->give_elemnodes (eid,nodes);
    
  rcn = new long [dofe[0]];
  reallocm (dofe[0],dofe[0],lsm);
  eldispl (lcid,eid,r.a);
    
  //  transformation of displacement vector
  long transf = Mt->locsystems (nodes);
  if (transf>0){
    reallocv (ndofe,aux);
    reallocm (ndofe,ndofe,tmat);
    transf_matrix (nodes,tmat);
    //locglobtransf (aux,r,tmat);
    lgvectortransf (aux,r,tmat);
    copyv (aux,r);
  }
 //  funkce transformujici x,y,z do roviny xy 
  tran_mat(x,y, tran, gx,  gy,  gz);
  glvectortransfblock (r, tran);
    
  rcn[0]=1;  rcn[1]=2;  rcn[2]=6;  rcn[3]=7;  rcn[4]=8;  rcn[5]=12;
  rcn[6]=13;  rcn[7]=14;  rcn[8]=18;  rcn[9]=19;  rcn[10]=20;  rcn[11]=24;
  for (i=0;i<Perlq->ndofe;i++){
    if (rcn[i]==0)  rs[i]=0.0;
    else rs[i]=r[rcn[i]-1];
  }
  //  contribution from q plane element with rotational degrees of freedom
  Perlq->ip_strains (lcid,eid,0,0,x,y,rs);
  
  /*
  vector eps(6);
  long ri=0;
  long ci=0;
  long ipp=Mt->elements[eid].ipp[ri][ci];
  for (long i=0;i<6;i++){
    eps[0]=Mm->ip[ipp].strain[0];
    eps[1]=Mm->ip[ipp].strain[1];
    eps[5]=Mm->ip[ipp].strain[2];
    loc_glob_tens_trans (eps,tran);
    Mm->ip[ipp].strain[0]=eps[0];
    Mm->ip[ipp].strain[1]=eps[1];
    Mm->ip[ipp].strain[2]=eps[5];
    ipp++;
  }
  */

  rcn[0]=3;  rcn[1]=4;  rcn[2]=5;  rcn[3]=9;  rcn[4]=10;  rcn[5]=11;
  rcn[6]=15;  rcn[7]=16;  rcn[8]=17;  rcn[9]=21;  rcn[10]=22;  rcn[11]=23;
  for (i=0;i<Q4pl->ndofe;i++){
    if (rcn[i]==0)  rd[i]=0.0;
    else rd[i]=r[rcn[i]-1];
  }
  //  contribution from q plate element
  Q4pl->ip_strains (lcid,eid,2,2,x,y,rd);
  
}


void shellq::nod_strains_ip (long lcid,long eid)
{
  
  //  contribution from triangular plane element with rotational degrees of freedom
  Perlq->nod_strains_ip (lcid,eid,0,0);
    
  //  contribution from triangular plate element
  Q4pl->nod_strains_ip (lcid,eid,2,2);
}
  
void shellq::strains (long lcid,long eid)
{
}

void shellq::res_ip_stresses (long lcid,long eid)
{
  Perlq->compute_nlstress (lcid,eid,0,0);
  Q4pl->compute_nlstress (lcid,eid,2,2);
}
  
void shellq::nod_stresses_ip (long lcid,long eid)
{
  
  //  contribution from triangular plane element with rotational degrees of freedom
  Perlq->nod_stresses_ip (lcid,eid,0,0);
    
  //  contribution from triangular plate element
  Q4pl->nod_stresses_ip (lcid,eid,2,2);
}



  
void shellq::stresses (long lcid,long eid)
{

}



/**
  The function computes initial values of the given quantities at each integration point of the
  element from the nodal values given by the parameter nodval. Initial condition types must be 
  the same for all nodes of the element.

  @param eid - element id
  @param ri  - block row index
  @param ci  - block column index
  @param nodval - nodal values of particular initial conditions.
                  nodval[i][j] represents value of j-th initial condition at i-th node of the given element.
  @param ictn - array of types of initial condition for each node of element.
                The type of initial condition determines which values are being specified in the node. 
                (ictn[i] & inistrain) returns nonzero if nodal values of initial strains are specified
                (ictn[i] & inistress) returns nonzero if nodal values of initial stresses are specified
                (ictn[i] & iniother)  returns nonzero if nodal values of initial values of eqother array are specified
                (ictn[i] & inicond)   returns nonzero if nodal values of other initial conditions are specified

  @retur The function does not return anything.

  Created by Tomas Koudelka 2004
  Revised by Tomas Koudelka 03.2012
*/
void shellq::inicipval(long eid, long ri, long ci, matrix &nodval, inictype *ictn)
{
  long i, j, k, l, ipp;
  long ii, jj, nv = nodval.n;
  double xi, eta;
//  double ipval;
  vector w, gp, anv(nne);
  long nstra, nstre, ncompstr, ncompeqother;
  long idstra, idstre, idoth, idic;
  inictype ict;

  nstra = idstra = nstre = idstre = idoth = idic = 0;

  ict = ictn[0];
  for (i=0; i<nne; i++)
  {
    if (ictn[i] != ict)
    {
      print_err("Incompatible types of initial conditions on element %ld\n"
                " at %ld. and %ld. nodes", __FILE__, __LINE__, __func__, eid+1, 1, i+1);
      abort();
    }
  }
  for (j = 0; j < nv; j++) // for all initial values
  {
    for(i = 0; i < nne; i++) // for all nodes on element
      anv[i] = nodval[i][j];
    for (ii = 0; ii < nb; ii++)
    {
      for (jj = 0; jj < nb; jj++)
      {
        ipp=Mt->elements[eid].ipp[ri+ii][ci+jj];
        if (intordsm[ii][jj] == 0)
          continue;
        reallocv (intordsm[ii][jj],gp);
        reallocv (intordsm[ii][jj],w);
        gauss_points (gp.a,w.a,intordsm[ii][jj]);
        for (k = 0; k < intordsm[ii][jj]; k++)
        {
          xi=gp[k];
          for (l = 0; l < intordsm[ii][jj]; l++)
          {
            eta=gp[l];
            //  value in integration point
//            ipval = approx (xi,eta,anv);
            ncompstr =  Mm->ip[ipp].ncompstr;
            ncompeqother = Mm->ip[ipp].ncompeqother;
            if ((ictn[0] & inistrain) && (j < ncompstr))
            {
              //Mm->ip[ipp].strain[idstra] += ipval;
              ipp++;
              continue;
            }
            if ((ictn[0] & inistress) && (j < nstra + ncompstr))
            {
              //Mm->ip[ipp].stress[idstre] += ipval;
              ipp++;
              continue;
            }
            if ((ictn[0] & iniother) && (j < nstra+nstre+ncompeqother))
            {
              //Mm->ip[ipp].eqother[idoth] += ipval;
              ipp++;
              continue;
            }
            if ((ictn[0] & inicond) && (j < nv))
            {
              if (Mm->ic[ipp] == NULL)
              {
                Mm->ic[ipp] = new double[nv-j];
                memset(Mm->ic[ipp], 0, sizeof(*Mm->ic[ipp])*(nv-j)); 
              }
              //Mm->ic[ipp][idic] += ipval;
              ipp++;
              continue;
            }
            ipp++;
          }
        }
      }
    }
    ipp=Mt->elements[eid].ipp[ri][ci];
    ncompstr =  Mm->ip[ipp].ncompstr;
    ncompeqother = Mm->ip[ipp].ncompeqother;
    if ((ictn[0] & inistrain) && (j < ncompstr))
    {
      nstra++;
      idstra++;
      continue;
    }
    if ((ictn[0] & inistress) && (j < nstra + ncompstr))
    {      
      nstre++;
      idstre++;
      continue;
    }  
    if ((ictn[0] & iniother)  && (j < nstra + nstre + ncompeqother))
    {
      idoth++;
      continue;
    }
    if ((ictn[0] & inicond) && (j < nv))
    {
      idic++;
      continue;
    }
  }
}
